@@ -73,7 +73,8 @@ typedef struct pa_daemon_conf {
         disallow_exit,
         log_meta,
         log_time,
-        flat_volumes;
+        flat_volumes,
+        lock_memory;
     int exit_idle_time,
         scache_idle_time,
         auto_log_target,
