@@ -51,12 +51,15 @@ PA_C_DECL_BEGIN
 /** The number of nanoseconds in a microsecond */
 #define PA_NSEC_PER_USEC ((unsigned long long) 1000ULL)
 
-/** Invalid time in usec */
+/** Invalid time in usec. \since 0.9.15 */
 #define PA_USEC_INVALID ((pa_usec_t) -1)
 
+/** Biggest time in usec. \since 0.9.18 */
+#define PA_USEC_MAX ((pa_usec_t) -2)
+
 struct timeval;
 
-/** Return the current timestamp, just like UNIX gettimeofday() */
+/** Return the current wallclock timestamp, just like UNIX gettimeofday(). */
 struct timeval *pa_gettimeofday(struct timeval *tv);
 
 /** Calculate the difference between the two specified timeval
