@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <pulse/rtclock.h>
 #include <pulse/timeval.h>
 #include <pulse/xmalloc.h>
 
@@ -37,37 +38,39 @@
 #include <pulsecore/macro.h>
 #include <pulsecore/refcnt.h>
 #include <pulsecore/flist.h>
+#include <pulsecore/core-rtclock.h>
 
 #include "pdispatch.h"
 
-/*#define DEBUG_OPCODES */
+/* #define DEBUG_OPCODES */
 
 #ifdef DEBUG_OPCODES
 
 static const char *command_names[PA_COMMAND_MAX] = {
+    /* Generic commands */
     [PA_COMMAND_ERROR] = "ERROR",
     [PA_COMMAND_TIMEOUT] = "TIMEOUT",
     [PA_COMMAND_REPLY] = "REPLY",
+
+    /* CLIENT->SERVER */
     [PA_COMMAND_CREATE_PLAYBACK_STREAM] = "CREATE_PLAYBACK_STREAM",
     [PA_COMMAND_DELETE_PLAYBACK_STREAM] = "DELETE_PLAYBACK_STREAM",
     [PA_COMMAND_CREATE_RECORD_STREAM] = "CREATE_RECORD_STREAM",
     [PA_COMMAND_DELETE_RECORD_STREAM] = "DELETE_RECORD_STREAM",
     [PA_COMMAND_AUTH] = "AUTH",
-    [PA_COMMAND_REQUEST] = "REQUEST",
     [PA_COMMAND_EXIT] = "EXIT",
     [PA_COMMAND_SET_CLIENT_NAME] = "SET_CLIENT_NAME",
     [PA_COMMAND_LOOKUP_SINK] = "LOOKUP_SINK",
     [PA_COMMAND_LOOKUP_SOURCE] = "LOOKUP_SOURCE",
     [PA_COMMAND_DRAIN_PLAYBACK_STREAM] = "DRAIN_PLAYBACK_STREAM",
-    [PA_COMMAND_PLAYBACK_STREAM_KILLED] = "PLAYBACK_STREAM_KILLED",
-    [PA_COMMAND_RECORD_STREAM_KILLED] = "RECORD_STREAM_KILLED",
     [PA_COMMAND_STAT] = "STAT",
-    [PA_COMMAND_GET_PLAYBACK_LATENCY] = "PLAYBACK_LATENCY",
+    [PA_COMMAND_GET_PLAYBACK_LATENCY] = "GET_PLAYBACK_LATENCY",
     [PA_COMMAND_CREATE_UPLOAD_STREAM] = "CREATE_UPLOAD_STREAM",
     [PA_COMMAND_DELETE_UPLOAD_STREAM] = "DELETE_UPLOAD_STREAM",
     [PA_COMMAND_FINISH_UPLOAD_STREAM] = "FINISH_UPLOAD_STREAM",
     [PA_COMMAND_PLAY_SAMPLE] = "PLAY_SAMPLE",
     [PA_COMMAND_REMOVE_SAMPLE] = "REMOVE_SAMPLE",
+
     [PA_COMMAND_GET_SERVER_INFO] = "GET_SERVER_INFO",
     [PA_COMMAND_GET_SINK_INFO] = "GET_SINK_INFO",
     [PA_COMMAND_GET_SINK_INFO_LIST] = "GET_SINK_INFO_LIST",
@@ -84,15 +87,104 @@ static const char *command_names[PA_COMMAND_MAX] = {
     [PA_COMMAND_GET_SOURCE_OUTPUT_INFO] = "GET_SOURCE_OUTPUT_INFO",
     [PA_COMMAND_GET_SOURCE_OUTPUT_INFO_LIST] = "GET_SOURCE_OUTPUT_INFO_LIST",
     [PA_COMMAND_SUBSCRIBE] = "SUBSCRIBE",
-    [PA_COMMAND_SUBSCRIBE_EVENT] = "SUBSCRIBE_EVENT",
+
     [PA_COMMAND_SET_SINK_VOLUME] = "SET_SINK_VOLUME",
     [PA_COMMAND_SET_SINK_INPUT_VOLUME] = "SET_SINK_INPUT_VOLUME",
     [PA_COMMAND_SET_SOURCE_VOLUME] = "SET_SOURCE_VOLME",
+
+    [PA_COMMAND_SET_SINK_MUTE] = "SET_SINK_MUTE",
+    [PA_COMMAND_SET_SOURCE_MUTE] = "SET_SOURCE_MUTE",
+
     [PA_COMMAND_TRIGGER_PLAYBACK_STREAM] = "TRIGGER_PLAYBACK_STREAM",
     [PA_COMMAND_FLUSH_PLAYBACK_STREAM] = "FLUSH_PLAYBACK_STREAM",
     [PA_COMMAND_CORK_PLAYBACK_STREAM] = "CORK_PLAYBACK_STREAM",
-    [PA_COMMAND_GET_AUTOLOAD_INFO] = "GET_AUTOLOAD_INFO",
-    [PA_COMMAND_GET_AUTOLOAD_INFO_LIST] = "GET_AUTOLOAD_INFO_LIST",
+
+    [PA_COMMAND_SET_DEFAULT_SINK] = "SET_DEFAULT_SINK",
+    [PA_COMMAND_SET_DEFAULT_SOURCE] = "SET_DEFAULT_SOURCE",
+
+    [PA_COMMAND_SET_PLAYBACK_STREAM_NAME] = "SET_PLAYBACK_STREAM_NAME",
+    [PA_COMMAND_SET_RECORD_STREAM_NAME] = "SET_RECORD_STREAM_NAME",
+
+    [PA_COMMAND_KILL_CLIENT] = "KILL_CLIENT",
+    [PA_COMMAND_KILL_SINK_INPUT] = "KILL_SINK_INPUT",
+    [PA_COMMAND_KILL_SOURCE_OUTPUT] = "SOURCE_OUTPUT",
+
+    [PA_COMMAND_LOAD_MODULE] = "LOAD_MODULE",
+    [PA_COMMAND_UNLOAD_MODULE] = "UNLOAD_MODULE",
+
+    [PA_COMMAND_ADD_AUTOLOAD___OBSOLETE] = "ADD_AUTOLOAD (obsolete)",
+    [PA_COMMAND_REMOVE_AUTOLOAD___OBSOLETE] = "REMOVE_AUTOLOAD (obsolete)",
+    [PA_COMMAND_GET_AUTOLOAD_INFO___OBSOLETE] = "GET_AUTOLOAD_INFO (obsolete)",
+    [PA_COMMAND_GET_AUTOLOAD_INFO_LIST___OBSOLETE] = "GET_AUTOLOAD_INFO_LIST (obsolete)",
+
+    [PA_COMMAND_GET_RECORD_LATENCY] = "GET_RECORD_LATENCY",
+    [PA_COMMAND_CORK_RECORD_STREAM] = "CORK_RECORD_STREAM",
+    [PA_COMMAND_FLUSH_RECORD_STREAM] = "FLUSH_RECORD_STREAM",
+    [PA_COMMAND_PREBUF_PLAYBACK_STREAM] = "PREBUF_PLAYBACK_STREAM",
+
+    /* SERVER->CLIENT */
+    [PA_COMMAND_REQUEST] = "REQUEST",
+    [PA_COMMAND_OVERFLOW] = "OVERFLOW",
+    [PA_COMMAND_UNDERFLOW] = "UNDERFLOW",
+    [PA_COMMAND_PLAYBACK_STREAM_KILLED] = "PLAYBACK_STREAM_KILLED",
+    [PA_COMMAND_RECORD_STREAM_KILLED] = "RECORD_STREAM_KILLED",
+    [PA_COMMAND_SUBSCRIBE_EVENT] = "SUBSCRIBE_EVENT",
+
+    /* A few more client->server commands */
+
+    /* Supported since protocol v10 (0.9.5) */
+    [PA_COMMAND_MOVE_SINK_INPUT] = "MOVE_SINK_INPUT",
+    [PA_COMMAND_MOVE_SOURCE_OUTPUT] = "MOVE_SOURCE_OUTPUT",
+
+    /* Supported since protocol v11 (0.9.7) */
+    [PA_COMMAND_SET_SINK_INPUT_MUTE] = "SET_SINK_INPUT_MUTE",
+
+    [PA_COMMAND_SUSPEND_SINK] = "SUSPEND_SINK",
+    [PA_COMMAND_SUSPEND_SOURCE] = "SUSPEND_SOURCE",
+
+    /* Supported since protocol v12 (0.9.8) */
+    [PA_COMMAND_SET_PLAYBACK_STREAM_BUFFER_ATTR] = "SET_PLAYBACK_STREAM_BUFFER_ATTR",
+    [PA_COMMAND_SET_RECORD_STREAM_BUFFER_ATTR] = "SET_RECORD_STREAM_BUFFER_ATTR",
+
+    [PA_COMMAND_UPDATE_PLAYBACK_STREAM_SAMPLE_RATE] = "UPDATE_PLAYBACK_STREAM_SAMPLE_RATE",
+    [PA_COMMAND_UPDATE_RECORD_STREAM_SAMPLE_RATE] = "UPDATE_RECORD_STREAM_SAMPLE_RATE",
+
+    /* SERVER->CLIENT */
+    [PA_COMMAND_PLAYBACK_STREAM_SUSPENDED] = "PLAYBACK_STREAM_SUSPENDED",
+    [PA_COMMAND_RECORD_STREAM_SUSPENDED] = "RECORD_STREAM_SUSPENDED",
+    [PA_COMMAND_PLAYBACK_STREAM_MOVED] = "PLAYBACK_STREAM_MOVED",
+    [PA_COMMAND_RECORD_STREAM_MOVED] = "RECORD_STREAM_MOVED",
+
+    /* Supported since protocol v13 (0.9.11) */
+    [PA_COMMAND_UPDATE_RECORD_STREAM_PROPLIST] = "UPDATE_RECORD_STREAM_PROPLIST",
+    [PA_COMMAND_UPDATE_PLAYBACK_STREAM_PROPLIST] = "UPDATE_RECORD_STREAM_PROPLIST",
+    [PA_COMMAND_UPDATE_CLIENT_PROPLIST] = "UPDATE_CLIENT_PROPLIST",
+    [PA_COMMAND_REMOVE_RECORD_STREAM_PROPLIST] = "REMOVE_RECORD_STREAM_PROPLIST",
+    [PA_COMMAND_REMOVE_PLAYBACK_STREAM_PROPLIST] = "REMOVE_PLAYBACK_STREAM_PROPLIST",
+    [PA_COMMAND_REMOVE_CLIENT_PROPLIST] = "REMOVE_CLIENT_PROPLIST",
+
+    /* SERVER->CLIENT */
+    [PA_COMMAND_STARTED] = "STARTED",
+
+    /* Supported since protocol v14 (0.9.12) */
+    [PA_COMMAND_EXTENSION] = "EXTENSION",
+
+    /* Supported since protocol v15 (0.9.15) */
+    [PA_COMMAND_GET_CARD_INFO] = "GET_CARD_INFO",
+    [PA_COMMAND_GET_CARD_INFO_LIST] = "GET_CARD_INFO_LIST",
+    [PA_COMMAND_SET_CARD_PROFILE] = "SET_CARD_PROFILE",
+
+    [PA_COMMAND_CLIENT_EVENT] = "GET_CLIENT_EVENT",
+    [PA_COMMAND_PLAYBACK_STREAM_EVENT] = "PLAYBACK_STREAM_EVENT",
+    [PA_COMMAND_RECORD_STREAM_EVENT] = "RECORD_STREAM_EVENT",
+
+    /* SERVER->CLIENT */
+    [PA_COMMAND_PLAYBACK_BUFFER_ATTR_CHANGED] = "PLAYBACK_BUFFER_ATTR_CHANGED",
+    [PA_COMMAND_RECORD_BUFFER_ATTR_CHANGED] = "RECORD_BUFFER_ATTR_CHANGED",
+
+    /* Supported since protocol v16 (0.9.16) */
+    [PA_COMMAND_SET_SINK_PORT] = "SET_SINK_PORT",
+    [PA_COMMAND_SET_SOURCE_PORT] = "SET_SOURCE_PORT"
 };
 
 #endif
@@ -115,9 +207,10 @@ struct pa_pdispatch {
     const pa_pdispatch_cb_t *callback_table;
     unsigned n_commands;
     PA_LLIST_HEAD(struct reply_info, replies);
-    pa_pdispatch_drain_callback drain_callback;
+    pa_pdispatch_drain_cb_t drain_callback;
     void *drain_userdata;
     const pa_creds *creds;
+    pa_bool_t use_rtclock;
 };
 
 static void reply_info_free(struct reply_info *r) {
@@ -134,21 +227,19 @@ static void reply_info_free(struct reply_info *r) {
         pa_xfree(r);
 }
 
-pa_pdispatch* pa_pdispatch_new(pa_mainloop_api *mainloop, const pa_pdispatch_cb_t*table, unsigned entries) {
+pa_pdispatch* pa_pdispatch_new(pa_mainloop_api *mainloop, pa_bool_t use_rtclock, const pa_pdispatch_cb_t *table, unsigned entries) {
     pa_pdispatch *pd;
-    pa_assert(mainloop);
 
+    pa_assert(mainloop);
     pa_assert((entries && table) || (!entries && !table));
 
-    pd = pa_xnew(pa_pdispatch, 1);
+    pd = pa_xnew0(pa_pdispatch, 1);
     PA_REFCNT_INIT(pd);
     pd->mainloop = mainloop;
     pd->callback_table = table;
     pd->n_commands = entries;
     PA_LLIST_HEAD_INIT(struct reply_info, pd->replies);
-    pd->drain_callback = NULL;
-    pd->drain_userdata = NULL;
-    pd->creds = NULL;
+    pd->use_rtclock = use_rtclock;
 
     return pd;
 }
@@ -213,11 +304,12 @@ int pa_pdispatch_run(pa_pdispatch *pd, pa_packet*packet, const pa_creds *creds,
 #ifdef DEBUG_OPCODES
 {
     char t[256];
-    char const *p;
-    if (!(p = command_names[command]))
+    char const *p = NULL;
+
+    if (command >= PA_COMMAND_MAX || !(p = command_names[command]))
         pa_snprintf((char*) (p = t), sizeof(t), "%u", command);
 
-    pa_log("Recieved opcode <%s>", p);
+    pa_log("[%p] Received opcode <%s>", pd, p);
 }
 #endif
 
@@ -226,7 +318,7 @@ int pa_pdispatch_run(pa_pdispatch *pd, pa_packet*packet, const pa_creds *creds,
     if (command == PA_COMMAND_ERROR || command == PA_COMMAND_REPLY) {
         struct reply_info *r;
 
-        for (r = pd->replies; r; r = r->next)
+        PA_LLIST_FOREACH(r, pd->replies)
             if (r->tag == tag)
                 break;
 
@@ -234,11 +326,11 @@ int pa_pdispatch_run(pa_pdispatch *pd, pa_packet*packet, const pa_creds *creds,
             run_action(pd, r, command, ts);
 
     } else if (pd->callback_table && (command < pd->n_commands) && pd->callback_table[command]) {
-        const pa_pdispatch_cb_t *c = pd->callback_table+command;
+        const pa_pdispatch_cb_t *cb = pd->callback_table+command;
 
-        (*c)(pd, command, tag, ts, userdata);
+        (*cb)(pd, command, tag, ts, userdata);
     } else {
-        pa_log("Recieved unsupported command %u", command);
+        pa_log("Received unsupported command %u", command);
         goto finish;
     }
 
@@ -255,7 +347,7 @@ finish:
     return ret;
 }
 
-static void timeout_callback(pa_mainloop_api*m, pa_time_event*e, const struct timeval *tv, void *userdata) {
+static void timeout_callback(pa_mainloop_api*m, pa_time_event*e, const struct timeval *t, void *userdata) {
     struct reply_info*r = userdata;
 
     pa_assert(r);
@@ -284,10 +376,9 @@ void pa_pdispatch_register_reply(pa_pdispatch *pd, uint32_t tag, int timeout, pa
     r->free_cb = free_cb;
     r->tag = tag;
 
-    pa_gettimeofday(&tv);
-    tv.tv_sec += timeout;
-
-    pa_assert_se(r->time_event = pd->mainloop->time_new(pd->mainloop, &tv, timeout_callback, r));
+    pa_assert_se(r->time_event = pd->mainloop->time_new(pd->mainloop,
+                                                        pa_timeval_rtstore(&tv, pa_rtclock_now() + timeout * PA_USEC_PER_SEC, pd->use_rtclock),
+                                                        timeout_callback, r));
 
     PA_LLIST_PREPEND(struct reply_info, pd->replies, r);
 }
@@ -299,7 +390,7 @@ int pa_pdispatch_is_pending(pa_pdispatch *pd) {
     return !!pd->replies;
 }
 
-void pa_pdispatch_set_drain_callback(pa_pdispatch *pd, void (*cb)(pa_pdispatch *pd, void *userdata), void *userdata) {
+void pa_pdispatch_set_drain_callback(pa_pdispatch *pd, pa_pdispatch_drain_cb_t cb, void *userdata) {
     pa_assert(pd);
     pa_assert(PA_REFCNT_VALUE(pd) >= 1);
     pa_assert(!cb || pa_pdispatch_is_pending(pd));
@@ -314,12 +405,9 @@ void pa_pdispatch_unregister_reply(pa_pdispatch *pd, void *userdata) {
     pa_assert(pd);
     pa_assert(PA_REFCNT_VALUE(pd) >= 1);
 
-    for (r = pd->replies; r; r = n) {
-        n = r->next;
-
+    PA_LLIST_FOREACH_SAFE(r, n, pd->replies)
         if (r->userdata == userdata)
             reply_info_free(r);
-    }
 }
 
 void pa_pdispatch_unref(pa_pdispatch *pd) {
